// Queue class

function Queue ( ) {

	var self = this;

	this.reset = function() {
		this.busy = false;
		this.items = new Array();
		this.froms = new Array();
		this.oneByOne = false;
		this.delay = 0;
	}
	
	this.reset();

	this.add = function(delivery) {
		this.items.push(delivery);
		if (!this.oneByOne) {
			if (delivery.passRequired || delivery.oneByOne) {
				var checkValue = (delivery.passRequired) ? delivery.extra : delivery.no;
				if (this.froms[delivery.from]) {
					if (this.froms[delivery.from] != checkValue) this.oneByOne = true;
				} else this.froms[delivery.from] = checkValue;
			}
		}
	}

	this.update = function() {
		if (this.items.length) {
			this.busy = true;
			if (this.oneByOne || this.delay) {
				var delivery = this.items.shift();
				delivery.updateStatus();
				if (!this.oneByOne) setTimeout(function(){ self.update() },this.delay);
				if (!this.items.length) this.reset();
			} else {
				do {
					var delivery = this.items.shift();
					delivery.updateStatus();
				} while (this.items.length);
				this.reset();
			}
		}
	}

	this.updateIfIdle = function() {
		if (!this.busy) this.update();
	}

	this.count = function() {
		return this.items.length;
	}

}

// Delivery class

function Delivery ( settings ) {

	var self = this;
	this.req = new Array();

	// Functions
	this.setDelivery = setDelivery;
	this.setFrom = setFrom;
	this.loadPage = loadPage;
	this.loadCookiePage = loadCookiePage;
	this.loadSection = loadSection;
	this.resetRequests = resetRequests;
	this.updateStatus = updateStatus;
	this.connectFailed = connectFailed;
	this.notFound = notFound;
	this.updateItems = updateItems;
	this.parseDate = parseDate;
	this.cleanDate = cleanDate;
	this.cleanLocation = cleanLocation;
	this.showMessage = showMessage;
	this.saveItems = saveItems;

	if (!iPhone) {
		this.loading = false;
		this.spinAngle = 0;
		this.startLoad = startLoad;
		this.stopLoad = stopLoad;
		this.addButtons = addButtons;
		this.editParent = editParent;
		this.closeParent = closeParent;
	}
	this.setDelivery(settings);

}

function setDelivery ( settings ) {

	this.reqCount = 0;
	this.reqComplete = 0;

	//  Success values:
	// -1: Couldn't connect, no internet
	//  0: Couldn't connect, site down
	//  1: Not found
	//  2: Found

	this.success = 2;
	this.lastUpdate = 0;
	this.nextUpdate = 0;
	this.interval = 0;
	this.items = new Array();
	this.previousItems = false;
	this.count = 0;

	this.fromName = 'Amazon.com';
	this.fromTrans = 'Amazon.com';
	this.fromEdited = false;
	this.encoding = '';
	this.extraRequired = false;
	this.passRequired = false;
	this.oneByOne = false;
	this.icon = '';

	this.url = '';
	this.html = '';
	this.tries = 0;

	// Initialization
	this.id = settings['uuid'];
	this.name = (settings['name']) ? settings['name'] : '';
	this.from = (settings['from']) ? settings['from'] : 'www.amazon.com';
	this.extra = (settings['extra']) ? settings['extra'] : '';
	this.no = (settings['no']) ? settings['no'] : '';

	this.date = false;
	if (settings['date']) this.date = stringToTimeStamp(settings['date']);

	this.html = (settings['html']) ? settings['html'] : '';
	this.htmldate = (settings['htmldate']) ? settings['htmldate'] : '';
	this.displayed = (settings['item']) ? settings['item'] : 0;

	if (!iPhone) {
		if (this.extra) this.password = getPassword(this.from,this.extra);
	} else this.password = (settings['password']) ? settings['password'] : 0;

	this.tap = false;

	switch (this.from) {
		case 'track.dhl-usa.com':
			this.from = 'www.dhl.com';
			break;
		case 'spportal.fedex.com':
			this.from = 'www.fedex.com';
			break;
	}

	this.domain = this.from;
	this.fromName = getFromName(this.from,false,false);
	this.fromTrans = getFromName(this.from,true,false);
	this.setFrom();

}

function loadPage (thisUrl,theFunction,theData,theArgs) {

	var thisReq = this.reqCount;
	this.reqCount++;
	this.req[thisReq] = new XMLHttpRequest();
	if (!this.sync) this.req[thisReq].delivery = this;
	this.req[thisReq].args = theArgs;
	this.req[thisReq].getTracker = getTracker;
	this.req[thisReq].onload = theFunction;

	var thisRequest = this.req[thisReq];
	this.req[thisReq].onreadystatechange = function(){
		if (this.timer && this.readyState >= 3) clearTimeout(this.timer);
		if (this.readyState == 4) {
			if (this.status == 0 && !this.responseText) {
				if (!iPhone && !thisRequest.delivery) {
					syncFail(theArgs);
				} else {
					this.abort(); // Stops onload from getting called in 10.5
					this.delivery.connectFailed();
				}
			}
		}
	}

	if (theData) {
		this.req[thisReq].open('POST',thisUrl);
		this.req[thisReq].setRequestHeader('Content-type','application/x-www-form-urlencoded');
		this.req[thisReq].setRequestHeader('Cache-Control','no-cache');
		this.req[thisReq].setRequestHeader('Content-length',theData.length);
		this.req[thisReq].setRequestHeader('Connection','close');
		this.req[thisReq].send(theData);
	} else {
		this.req[thisReq].open('GET',thisUrl);
		this.req[thisReq].setRequestHeader('Cache-Control','no-cache');
		if (this.encoding != '') {
			this.req[thisReq].setRequestHeader('Content-type','text/html; charset='+this.encoding); 
			this.req[thisReq].overrideMimeType('text/html; charset='+this.encoding);
		}
		this.req[thisReq].send();
	}

	if (iPhone) {
		this.req[thisReq].timer = setTimeout(function(){
			thisRequest.abort();
			thisRequest.delivery.connectFailed();
		},30000);
	} else if (this.sync) {
		this.req[thisReq].timer = setTimeout(function(){
			syncFail(theArgs);
		},15000);
	}

}

function DummyRequest (delivery, responseText, args) {
	this.delivery = delivery;
	this.responseText = responseText;
	this.status = 200;
	this.args = args;
	this.abort = function(){};
	this.getTracker = getTracker;
	this.getTracker();
}

function loadSection (theSection,theArgs) {
	var thisReq = this.reqCount;
	this.reqCount++;
	this.req[thisReq] = new DummyRequest(this,theSection,theArgs);
}

function resetRequests (cancelAll) {
	if (cancelAll) {
		for (i in this.req) {
			if (this.req[i]) {
				this.req[i].abort();
				clearTimeout(this.req[i].timer);
			}
		}
	}
	this.reqCount = 0;
	this.reqComplete = 0;
}

function connectFailed () {
	if (!this.count) this.count = -1;
	this.notFound(false,true);
	this.reqComplete++;
	this.updateItems();
}

function notFound (thisTitle,connectionDown) {

	if (!iPhone) this.stopLoad();
	if (this.count == 0) {
	
		this.count = -1;
		this.items = new Array();
		this.items[0] = resetItem();

		this.items[0]['icon'] = 'error';
		this.items[0]['theItem'] = (this.name)?htmlEntities(this.name):htmlEntities(this.no);
		this.items[0]['shipDate']  = thisTitle ? (thisTitle) : translate('Not found');
		this.items[0]['deliveryDate'] = translate('Please check your settings.');
		this.items[0]['shipDateBold'] = this.items[0]['shipDaysBold'] = true;
		this.items[0]['deliveryDateBold'] = this.items[0]['deliveryDaysBold'] = false;

		if (this.passRequired && (!this.password || this.password == ' ')) {
			this.items[0]['shipDate'] = translate('Password required');
			if (iPhone) {
				this.tap = 'password';
				this.items[0]['deliveryDate'] = translate('Tap to enter your password.');
			} else {
				this.items[0]['deliveryDate'] = '<a href="javascript:editPassword(\''+this.id+'\');">'+
					translate('Click to enter your password.')+'</a>';
				this.items[0]['deliveryDateBold'] = this.items[0]['deliveryDaysBold'] = true;
				this.items[0]['clickable'] = true;
			}
		}

		this.items[0]['shipDays'] = this.items[0]['shipDate'];
		this.items[0]['deliveryDays'] = this.items[0]['deliveryDate'];
		this.success = 1;
		this.displayed = 0;

	} else if (this.count == -1) {

		this.items = new Array();
		this.items[0] = resetItem();

		this.items[0]['icon'] = 'error';
		this.items[0]['theItem'] = (this.name)?htmlEntities(this.name):htmlEntities(this.no);
		this.items[0]['shipDate'] = translate('Couldn’t connect'); 
		this.items[0]['deliveryDate'] = this.fromTrans+' '+translate('is not available.'); 
		this.items[0]['shipDays'] = this.items[0]['shipDate'];
		this.items[0]['shipDateBold'] = this.items[0]['shipDaysBold'] = true;
		this.items[0]['deliveryDays'] = this.items[0]['deliveryDate'];
		this.items[0]['deliveryDateBold'] = this.items[0]['deliveryDaysBold'] = false;
		this.success = (connectionDown) ? -1 : 0;
		this.displayed = 0;

	} else this.success = (connectionDown) ? -1 : 0;

	if (!iPhone) checkVersion(1,true);

}

// End of Delivery class

function getOther () {

	var delivery = this.delivery;
	if (!iPhone) delivery.oldhtml = delivery.html;

	if ((this.status == 200 || this.status == 0) && this.responseText) {
		delivery.html = stripWhiteSpace(this.responseText);
	} else {
		if (!delivery.count) delivery.count = -1;
		delivery.notFound();
	}

	if (!iPhone) updateOther(delivery);

	delivery.reqComplete++;
	delivery.updateItems();
	return;

}

function updateItems () {

	if (this.reqComplete >= this.reqCount) {

		this.previousItems = false;

		var newInterval = 1440;
		if (this.success == -1) { // Couldn't connect, no internet
			newInterval = 0;
		} else if (this.success <= 1) { // Not found or couldn't connect, site down
			newInterval = 60;
		}

		if (this.from == 'other') {

			if (newInterval > 60) newInterval = 60;
			if (this.success > 0) this.lastUpdate = new Date();

		} else if (this.count > 0) {

			for (var x in this.items) {
				var thisItem = this.items[x];
				var theResult = false;
				var hasDeliveryDays = false;
	
				thisItem['theDetails']['date'] = stripHTML(thisItem['theDetails']['date']);
				thisItem['theDetails']['location'] = stripHTML(thisItem['theDetails']['location']);
				thisItem['theDetails']['status'] = stripHTML(thisItem['theDetails']['status']);

				// Update the display
				if (thisItem['shipSaved'] || !thisItem['shipDate']) {
					theResult = this.parseDate(thisItem['shipSaved'],thisItem['theStatus'],thisItem['theDetails'],
						thisItem['trackingUrl'],'Shipped');
					thisItem['shipDate'] = theResult[0];
					thisItem['shipDays'] = theResult[1];
					thisItem['daysNumber'] = theResult[2];
					thisItem['shipStamp'] = theResult[3];
					thisItem['shipDateBold'] = theResult[4];
					thisItem['shipDaysBold'] = theResult[5];
					thisItem['statusDate'] = (theResult[6]) ? theResult[6] : theResult[3];					
					thisItem['date'] = theResult[3];
				}
				if (thisItem['deliverySaved'] || !thisItem['deliveryDate']) {
					theResult = this.parseDate(thisItem['deliverySaved'],thisItem['theStatus'],false,
						thisItem['trackingUrl'],'Delivered');
					thisItem['deliveryDate'] = theResult[0];
					thisItem['deliveryDays'] = theResult[1];
					thisItem['deliveryStamp'] = theResult[3];
					thisItem['deliveryDateBold'] = theResult[4];
					thisItem['deliveryDaysBold'] = theResult[5];
					if (theResult[2] !== false && theResult[2] >= 0) {
						thisItem['daysNumber'] = theResult[2];
						hasDeliveryDays = true;
					}
					if (theResult[3]) thisItem['date'] = theResult[3];
				}

				if (thisItem['theStatus'].match(/^complete$/i)) {
					thisItem['icon'] = 'checkmark';
				} else if (thisItem['daysNumber'] > 999) {
					thisItem['icon'] = '';	
				} else if (thisItem['daysNumber'] > 99) {
					thisItem['icon'] = '';	
				} else if (thisItem['daysNumber'] > 0 || (thisItem['daysNumber'] === 0 && trim(thisItem['deliverySaved']))) {
					thisItem['icon'] = '';
				} else if (thisItem['theStatus'] || thisItem['theDetails']['status']) {
					thisItem['icon'] = 'transit';
				} else {
					thisItem['icon'] = 'error';
				}

				var thisInterval = getInterval(thisItem['icon'],thisItem['daysNumber'],hasDeliveryDays);
				if (thisInterval < newInterval) newInterval = thisInterval;

				this.items[x] = thisItem;
			}
			if (this.success > 0) this.lastUpdate = new Date();

		}

		this.resetRequests(false);
		this.interval = newInterval;

		if (iPhone) {
			this.saveItems(this.displayed,this.items);
		} else {
			this.nextUpdate = ((new Date()).getTime()/1000)+(this.interval * 60);
			doneCount++;
			if (this.success > 0) {
				hadSuccess = true;
				if (this.success == 2) setHistoryTitle(this.id);
			}
			showTime(true);
			checkVersion(7,false);
			displayItem(this.id,this.displayed,false);
			if (growlEnabled || pushEnabled) showNotifications(this);
			this.saveItems();
		}
		if (updateQueue) updateQueue.update();

	}

}

function getInterval (icon,daysApart,hasDeliveryDays) {
	var thisInterval = 60;
	if (icon == 'checkmark') {
		thisInterval = 1440;
	} else if (hasDeliveryDays && daysApart <= -3) {
		thisInterval = 180;
	} else if (daysApart <= 3) {
		thisInterval = 20;
	} else if (daysApart <= 10) {
		thisInterval = 60;
	} else if (daysApart > 10) {
		thisInterval = 180;
	}
	return thisInterval;
}

function resetItem () {

	var thisItem = new Array();
	thisItem['theItem'] = translate('Unknown Item');
	thisItem['theStatus'] = '';

	thisItem['theDetails'] = new Array();
	thisItem['theDetails']['date'] = '';
	thisItem['theDetails']['location'] = '';
	thisItem['theDetails']['status'] = '';

	thisItem['shipSaved'] = '';
	thisItem['shipDate'] = ''; 
	thisItem['shipDays'] = '';

	thisItem['deliverySaved'] = '';
	thisItem['deliveryDate'] = '';
	thisItem['deliveryDays'] = '';
	thisItem['clickable'] = false;

	thisItem['daysNumber'] = false;
	thisItem['trackingNo'] = false;

	return thisItem;

}

// Helper functions

function translate (key) {
	try {
		var ret = translation[key];
		if (ret === undefined) ret = key;
		return ret;
	} catch (ex) { }
	return key;
}

function translateFormat (key,value1,value2,value3) {
	key = translate(key);
	if (value1) key = key.replace('%@',value1);
	if (value2) key = key.replace('%@',value2);
	if (value3) key = key.replace('%@',value3);
	return key;
}

function fullEscape (theString) {
	theString = escape(theString);
	theString = theString.replace(/@/g,'%40');
	theString = theString.replace(/\+/g,'%2B');
	return theString;
}

function getUnixTime () {
	var currentTime = new Date();
	return currentTime.getTime()/1000;
}

// Display text

function stripWhiteSpace (theString) {
	return (typeof theString == 'string') ? theString.replace(/(\s|\n)+/g,' ') : '';
}

function trim (theString) {
	if (typeof theString == 'string') {
		// Remove non-breaking space characters (&#160; or \xAO)
		while (theString.indexOf(' ') > -1) {
			theString = theString.replace(' ',' ');
		}
		theString = theString.replace(/^(\s|&nbsp;)+|(\s|&nbsp;)+$/ig,'');
	}
	return theString;
}

function stripHTML (theString) {
	if (typeof theString == 'string') {

		// Deal with some really ugly HTML from UPS Mail Innovations
		theString = theString.replace(/<a [^>]*onmouseover="([^"]*)"[^>]*>/g,' ');

		// Remove script, noscript, and style tags
		theString = theString.replace(/<(noscript|script|style) ?[^>]*>(.*?)<\/\1>/ig,' ');

		// Remove HTML tags
		theString = theString.replace(/<[^>]*>/g,' ');

		// Remove non-breaking space characters (&#160; or \xAO) and reduce mutiple spaces
		while (theString.indexOf(' ') > -1) theString = theString.replace(' ',' ');
		theString = theString.replace(/(&nbsp;|\s)+/g,' ');

		// Convert some HTML entities
		theString = theString.replace(/&quot;/g,'"');

	}
	return trim(theString);
}

function htmlEntities (theString) {
	if (typeof theString == 'string') {
		theString = theString.replace(/\&/g,'&#38;');
		theString = theString.replace(/"/g,'&quot;');
		theString = theString.replace(/</g,'&#60;');
		theString = theString.replace(/>/g,'&#62;');
	}
	return theString;
}

function decodeDecimalEntities ( theText ) {
	return theText.replace(/&#(\d+);/g, 
		function(whole,parens) {
			if (parens >= -32768 && parens <= 65535) {
				return String.fromCharCode(parens);
			} else return whole;
		});
}

function removeHtmlEntities (theString) {
	if (typeof theString == 'string') {
		theString = decodeDecimalEntities(theString);

		theString = theString.replace(/\&amp;/g,'&');
		theString = theString.replace(/&quot;/g,'"');
		theString = theString.replace(/&nbsp;/g,' ');
		theString = theString.replace(/&ndash;/g,'–');
		theString = theString.replace(/&mdash;/g,'—');

		theString = theString.replace(/\&auml;/g,'ä');
		theString = theString.replace(/\&euml;/g,'ë');
		theString = theString.replace(/\&iuml;/g,'ï');
		theString = theString.replace(/\&ouml;/g,'ö');
		theString = theString.replace(/\&uuml;/g,'ü');
		theString = theString.replace(/\&Auml;/g,'Ä');
		theString = theString.replace(/\&Euml;/g,'Ë');
		theString = theString.replace(/\&Iuml;/g,'Ï');
		theString = theString.replace(/\&Ouml;/g,'Ö');
		theString = theString.replace(/\&Uuml;/g,'Ü');

		theString = theString.replace(/\&aacute;/g,'á');
		theString = theString.replace(/\&eacute;/g,'é');
		theString = theString.replace(/\&iacute;/g,'í');
		theString = theString.replace(/\&oacute;/g,'ó');
		theString = theString.replace(/\&uacute;/g,'ú');
		theString = theString.replace(/\&Aacute;/g,'Á');
		theString = theString.replace(/\&Eacute;/g,'É');
		theString = theString.replace(/\&Iacute;/g,'Í');
		theString = theString.replace(/\&Oacute;/g,'Ó');
		theString = theString.replace(/\&Uacute;/g,'Ú');

		theString = theString.replace(/\&agrave;/g,'à');
		theString = theString.replace(/\&egrave;/g,'è');
		theString = theString.replace(/\&igrave;/g,'ì');
		theString = theString.replace(/\&ograve;/g,'ò');
		theString = theString.replace(/\&ugrave;/g,'ù');
		theString = theString.replace(/\&Agrave;/g,'À');
		theString = theString.replace(/\&Egrave;/g,'È');
		theString = theString.replace(/\&Igrave;/g,'Ì');
		theString = theString.replace(/\&Ograve;/g,'Ò');
		theString = theString.replace(/\&Ugrave;/g,'Ù');

		theString = theString.replace(/\&acirc;/g,'â');
		theString = theString.replace(/\&ecirc;/g,'ê');
		theString = theString.replace(/\&icirc;/g,'î');
		theString = theString.replace(/\&ocirc;/g,'ô');
		theString = theString.replace(/\&ucirc;/g,'û');
		theString = theString.replace(/\&Acirc;/g,'Â');
		theString = theString.replace(/\&Ecirc;/g,'Ê');
		theString = theString.replace(/\&Icirc;/g,'Î');
		theString = theString.replace(/\&Ocirc;/g,'Ô');
		theString = theString.replace(/\&Ucirc;/g,'Û');

		theString = theString.replace(/\&agrave;/g,'à');
		theString = theString.replace(/\&egrave;/g,'è');
		theString = theString.replace(/\&igrave;/g,'ì');
		theString = theString.replace(/\&ograve;/g,'ò');
		theString = theString.replace(/\&ugrave;/g,'ù');
		theString = theString.replace(/\&Agrave;/g,'À');
		theString = theString.replace(/\&Egrave;/g,'È');
		theString = theString.replace(/\&Igrave;/g,'Ì');
		theString = theString.replace(/\&Ograve;/g,'Ò');
		theString = theString.replace(/\&Ugrave;/g,'Ù');

		theString = theString.replace(/\&ntilde;/g,'ñ');
		theString = theString.replace(/\&atilde;/g,'ã');
		theString = theString.replace(/\&otilde;/g,'õ');
		theString = theString.replace(/\&Ntilde;/g,'Ñ');
		theString = theString.replace(/\&Atilde;/g,'Ã');
		theString = theString.replace(/\&Otilde;/g,'Õ');

		theString = theString.replace(/\&aring;/g,'å');
		theString = theString.replace(/\&Aring;/g,'Å');

		theString = theString.replace(/\&oslash;/g,'ø');
		theString = theString.replace(/\&Oslash;/g,'Ø');
		
		theString = theString.replace(/\&copy;/g,'©');
	}
	return theString;
}

function cleanOtherHTML (theString) {
	if (typeof theString == 'string') {
		theString = theString.replace(/<[^>]*FOOTER START[^>]*>(.*?)<[^>]*FOOTER END[^>]*>/i,' '); // interparcel.com
		theString = theString.replace(/<font class="secretTextNextGen"[^>]*>[^<]*<\/font>/i,' '); // dell.com
		theString = stripWhiteSpace(stripHTML(theString));
	}
	return theString;
}

// Parse dates

function stringToTimeStamp ( dateString ) {
	dateString = dateString.toString().replace(/-/g,'/');
	return new Date(dateString);
}

function parseDate (theInput,theStatus,theDetails,trackingUrl,theType) {

	var theOutput = new Array();
	var endDate = false;

	theOutput[0] = ''; // xDate
	theOutput[1] = ''; // xDays
	theOutput[2] = false; // daysNumber
	theOutput[3] = 0; // xStamp
	theOutput[4] = false; // xDateBold
	theOutput[5] = false; // xDaysBold
	theOutput[6] = 0; // statusDate

	if (theType == 'Shipped') {
		theOutput[0] = translate('Ship date unknown');
		theOutput[1] = translate('Ship date unknown');
	} else {
		theOutput[0] = translate('Delivery date unknown');
		theOutput[1] = translate('Delivery date unknown');
	}

	theInput = this.cleanDate(theInput);
	endDate = theInput[1];
	theInput = theInput[0];

	if (theStatus.match(/^complete$/i)) {

		theOutput[0] = translate(theType+'!');
		if (endDate) {
			theOutput[1] = translate(theType+' %@');
			theOutput[3] = convertDate(endDate);
		} else {
			theOutput[1] = translate(theType+'!');
		}
		theOutput[2] = 0;
		theOutput[4] = true;
		theOutput[5] = true;

	} else if (theInput) {

		if (endDate) {

			if (theType == 'Shipped' && theStatus.match(/^shipped$/i)) {
				theOutput[0] = translate(theType+'!');
				theOutput[1] = translate(theType+' %@');
				theOutput[4] = true;
			} else {
				theOutput[0] = theOutput[1] = translate(theType+' by %@');
				theOutput[4] = false;
			}

			theOutput[2] = getDaysApart(endDate);
			theOutput[3] = convertDate(endDate);
			theOutput[5] = false;

		} else {
			theOutput[0] = theOutput[1] = theInput;
			theOutput[4] = true;
		}

	} else if (theStatus.match(/^shipped$/i) && theType == 'Shipped') {

		theOutput[0] = theOutput[1] = translate('Shipped!');
		theOutput[4] = theOutput[5] = true;

	}

	if (theType == 'Shipped') {
		if (theDetails['status']) {
			if (!theDetails['status'].match(/^delivered$/i) || theDetails['location']) {

				theOutput[0] = theDetails['status'];
				if (theDetails['location']) theOutput[0] += ': '+theDetails['location'];

				var detailDate = this.cleanDate(theDetails['date']);
				detailDate = detailDate[1];
				if (detailDate) theOutput[1] = translate('Status updated %@');
				theOutput[4] = true;
				theOutput[6] = convertDate(detailDate);

			}
		} else if (theStatus && !theStatus.match(/^complete$/i) && !theStatus.match(/^shipped$/i)) {
			theOutput[0] = theStatus;
			theOutput[4] = true;
		}
	}

	return theOutput;

}

function splitDate (theInput) {
	theResult = theInput.match(/(.*) - ([^(]*)/i);
	if (theResult && theResult[2]) theInput = theResult[2];
	return theInput;
}

function cleanGermanDate (theInput) {
	theInput = theInput.replace('.',' ');
	theInput = theInput.replace(/\bJanuar\s/i,'January ');
	theInput = theInput.replace(/\bFebruar\s/i,'February ');
	theInput = theInput.replace(/\bMärz\s/i,'March ');
	theInput = theInput.replace(/\bMai\s/i,'May ');
	theInput = theInput.replace(/\bJuni\s/i,'June ');
	theInput = theInput.replace(/\bJuli\s/i,'July ');
	theInput = theInput.replace(/\bOktober\s/i,'October ');
	theInput = theInput.replace(/\bDezember\s/i,'December ');
	return theInput;
}

function cleanFrenchDate (theInput) {
	theInput = theInput.replace('.',' ');
	theInput = theInput.replace(/\bjanvier\s/i,'January ');
	theInput = theInput.replace(/\bfévrier\s/i,'February ');
	theInput = theInput.replace(/\bmars\s/i,'March ');
	theInput = theInput.replace(/\bavril\s/i,'April ');
	theInput = theInput.replace(/\bmai\s/i,'May ');
	theInput = theInput.replace(/\bjuin\s/i,'June ');
	theInput = theInput.replace(/\bjuillet\s/i,'July ');
	theInput = theInput.replace(/\baoût\s/i,'August ');
	theInput = theInput.replace(/\bseptembre\s/i,'September ');
	theInput = theInput.replace(/\boctobre\s/i,'October ');
	theInput = theInput.replace(/\bnovembre\s/i,'November ');
	theInput = theInput.replace(/\bdécembre\s/i,'December ');
	return theInput;
}

function cleanItalianDate (theInput) {
	theInput = theInput.replace(/\b(gen|genn)\.?\s/i,'January ');
	theInput = theInput.replace(/\b(feb|febbr)\.?\s/i,'February ');
	theInput = theInput.replace(/\b(mar)\.?\s/i,'March ');
	theInput = theInput.replace(/\b(apr)\.?\s/i,'April ');
	theInput = theInput.replace(/\b(mag|magg)\.?\s/i,'May ');
	theInput = theInput.replace(/\b(giu|giugno)\.?\s/i,'June ');
	theInput = theInput.replace(/\b(lug|luglio)\.?\s/i,'July ');
	theInput = theInput.replace(/\b(ago|ag)\.?\s/i,'August ');
	theInput = theInput.replace(/\b(set|sett)\.?\s/i,'September ');
	theInput = theInput.replace(/\b(ott)\.?\s/i,'October ');
	theInput = theInput.replace(/\b(nov)\.?\s/i,'November ');
	theInput = theInput.replace(/\b(dic)\.?\s/i,'December ');
	return theInput;
}

function cleanDanishDate (theInput) {
	theInput = theInput.replace('.',' ');
	theInput = theInput.replace(/\bjanuar\s/i,'January ');
	theInput = theInput.replace(/\bfebruar\s/i,'February ');
	theInput = theInput.replace(/\bmarts\s/i,'March ');
	theInput = theInput.replace(/\bmaj\s/i,'May ');
	theInput = theInput.replace(/\bjuni\s/i,'June ');
	theInput = theInput.replace(/\bjuli\s/i,'July ');
	theInput = theInput.replace(/\boktober\s/i,'October ');
	return theInput;
}

function getSwedishMonth (theInput) {
	switch (theInput) {
		case 'januari':		case 'jan':	return '1';
		case 'februari':	case 'feb':	return '2';
		case 'mars':		case 'mar':	return '3';
		case 'april':		case 'apr':	return '4';
		case 'maj':						return '5';
		case 'juni':		case 'jun':	return '6';
		case 'juli':		case 'jul':	return '7';
		case 'augusti':		case 'aug':	return '8';
		case 'september':	case 'sep':	return '9';
		case 'oktober':		case 'okt':	return '10';
		case 'november':	case 'mov':	return '11';
		case 'december':	case 'dec':	return '12';
	}
}

function getDaysApart ( theDate ) {
	var thisDate = new Date();
	thisDate.setTime(theDate);
	thisDate.setHours(0);
	thisDate.setMinutes(0);
	thisDate.setSeconds(0);
	return Math.ceil((thisDate-new Date())/86400000);
}